=== modified file '__init__.py'
--- __init__.py	2008-05-08 08:16:51 +0000
+++ __init__.py	2008-05-08 08:45:15 +0000
@@ -705,18 +705,17 @@
 
     def run(self, path):
         try:
-            from bzrlib.plugins.gtk.diff import (DiffWindow,
+            from bzrlib.plugins.gtk.diff import (DiffController,
                                                  MergeDirectiveController)
             lines = open(path, 'rb').readlines()
             lines = [l.replace('\r\n', '\n') for l in lines]
             try:
                 directive = merge_directive.MergeDirective.from_lines(lines)
             except errors.NotAMergeDirective:
-                window = DiffWindow()
-                window.set_diff_text(path, lines)
+                controller = DiffController(path, lines)
             else:
-                controller = MergeDirectiveController.for_gtk(directive, path)
-                window = controller.window
+                controller = MergeDirectiveController(path, directive)
+            window = controller.window
             window.show()
             gtk = self.open_display()
             window.connect("destroy", gtk.main_quit)

=== modified file 'diff.py'
--- diff.py	2008-05-08 08:16:51 +0000
+++ diff.py	2008-05-08 08:45:15 +0000
@@ -412,7 +412,7 @@
     differences between two revisions on a branch.
     """
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, operations=None):
         Window.__init__(self, parent)
         self.set_border_width(0)
         self.set_title("bzrk diff")
@@ -423,7 +423,7 @@
         width = int(monitor.width * 0.66)
         height = int(monitor.height * 0.66)
         self.set_default_size(width, height)
-
+        self.operations = operations
         self.construct()
 
     def construct(self):
@@ -443,7 +443,47 @@
 
         :return: None, meaning that no button bar will be used.
         """
-        return None
+        if self.operations is None:
+            return None
+        hbox = gtk.HButtonBox()
+        hbox.set_layout(gtk.BUTTONBOX_START)
+        for title, method in self.operations:
+            merge_button = gtk.Button(title)
+            merge_button.show()
+            merge_button.set_relief(gtk.RELIEF_NONE)
+            merge_button.connect("clicked", method)
+            hbox.pack_start(merge_button, expand=False, fill=True)
+        hbox.show()
+        return hbox
+
+    def _get_merge_target(self):
+        d = gtk.FileChooserDialog('Merge branch', self,
+                                  gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
+                                  buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK,
+                                           gtk.STOCK_CANCEL,
+                                           gtk.RESPONSE_CANCEL,))
+        try:
+            result = d.run()
+            if result != gtk.RESPONSE_OK:
+                raise SelectCancelled()
+            return d.get_current_folder_uri()
+        finally:
+            d.destroy()
+
+    def _get_save_path(self, basename):
+        d = gtk.FileChooserDialog('Save As', self,
+                                  gtk.FILE_CHOOSER_ACTION_SAVE,
+                                  buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK,
+                                           gtk.STOCK_CANCEL,
+                                           gtk.RESPONSE_CANCEL,))
+        d.set_current_name(basename)
+        try:
+            result = d.run()
+            if result != gtk.RESPONSE_OK:
+                raise SelectCancelled()
+            return urlutils.local_path_from_url(d.get_uri())
+        finally:
+            d.destroy()
 
     def set_diff_text(self, description, lines):
         """Set the diff from a text.
@@ -467,21 +507,45 @@
         self.diff.set_file(file_path)
 
 
-class MergeDirectiveController(object):
-
-    def __init__(self, directive, path):
+class DiffController(object):
+
+    def __init__(self, path, patch, window=None):
+        self.path = path
+        self.window = None
+        self.patch = patch
+        if window is None:
+            window = DiffWindow(operations=self._provide_operations())
+            self.initialize_window(window)
+
+    def initialize_window(self, window):
+        self.window = window
+        window.set_diff_text(self.path, self.patch)
+
+    def perform_save(self, window):
+        try:
+            save_path = self.window._get_save_path(osutils.basename(self.path))
+        except SelectCancelled:
+            return
+        source = open(self.path, 'rb')
+        try:
+            target = open(save_path, 'wb')
+            try:
+                osutils.pumpfile(source, target)
+            finally:
+                target.close()
+        finally:
+            source.close()
+
+    def _provide_operations(self):
+        return [('Save', self.perform_save)]
+
+
+class MergeDirectiveController(DiffController):
+
+    def __init__(self, path, directive):
+        DiffController.__init__(self, path, directive.patch.splitlines(True))
         self.directive = directive
-        self.path = path
         self.merge_target = None
-        self.window = None
-
-    @staticmethod
-    def for_gtk(directive, path):
-        controller = MergeDirectiveController(directive, path)
-        window = MergeDirectiveWindow(controller._provide_operations())
-        controller.window = window
-        window.set_diff_text(path, directive.patch.splitlines(True))
-        return controller
 
     def _provide_operations(self):
         return [('Merge', self.perform_merge), ('Save', self.perform_save)]
@@ -517,70 +581,6 @@
         finally:
             tree.unlock()
 
-    def perform_save(self, window):
-        try:
-            save_path = self.window._get_save_path(osutils.basename(self.path))
-        except SelectCancelled:
-            return
-        source = open(self.path, 'rb')
-        try:
-            target = open(save_path, 'wb')
-            try:
-                osutils.pumpfile(source, target)
-            finally:
-                target.close()
-        finally:
-            source.close()
-
-
-class MergeDirectiveWindow(DiffWindow):
-
-    def __init__(self, operations):
-        self.operations = operations
-        DiffWindow.__init__(self, None)
-
-    def _get_button_bar(self):
-        """The button bar has only the Merge button"""
-        hbox = gtk.HButtonBox()
-        hbox.set_layout(gtk.BUTTONBOX_START)
-        for title, method in self.operations:
-            merge_button = gtk.Button(title)
-            merge_button.show()
-            merge_button.set_relief(gtk.RELIEF_NONE)
-            merge_button.connect("clicked", method)
-            hbox.pack_start(merge_button, expand=False, fill=True)
-        hbox.show()
-        return hbox
-
-    def _get_merge_target(self):
-        d = gtk.FileChooserDialog('Merge branch', self,
-                                  gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
-                                  buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK,
-                                           gtk.STOCK_CANCEL,
-                                           gtk.RESPONSE_CANCEL,))
-        try:
-            result = d.run()
-            if result != gtk.RESPONSE_OK:
-                raise SelectCancelled()
-            return d.get_current_folder_uri()
-        finally:
-            d.destroy()
-
-    def _get_save_path(self, basename):
-        d = gtk.FileChooserDialog('Save As', self,
-                                  gtk.FILE_CHOOSER_ACTION_SAVE,
-                                  buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK,
-                                           gtk.STOCK_CANCEL,
-                                           gtk.RESPONSE_CANCEL,))
-        d.set_current_name(basename)
-        try:
-            result = d.run()
-            if result != gtk.RESPONSE_OK:
-                raise SelectCancelled()
-            return urlutils.local_path_from_url(d.get_uri())
-        finally:
-            d.destroy()
-
 
 def iter_changes_to_status(source, target):
     """Determine the differences between trees.

